package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element;

import java.util.HashMap;
public class PageAction extends Action {

    public String process (Element message) {
	
	Element request = (Element)message.getElementsByTagName("request").item(0);
	String info = request.getAttribute("info");
	// remove the page/ bit
	info = GSPath.removeFirstLink(info);
	String page_name = GSPath.getFirstLink(info);

	// create the return page tree
	Element page = doc_.createElement("page");
	page.setAttribute("lang", message.getAttribute("lang"));
	// add the lang stuff from xml_in
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, "translate"), true));
	// add the system stuff from xml_in
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, "config"), true));

	if (page_name.equals("")) {
	    // for now send an error. change to always display the home page?
	    return GSHTML.errorPage(" page action specified, but no page specified");
	}
	if (page_name.equals("home")) {
	    return homePage(page, request);
	} else if (page_name.equals("about")) {
	    return aboutPage(page, request);
	} else {
	    return GSHTML.errorPage("page: "+page_name+" not yet implemented");
	}
    }


    protected String homePage(Element page, Element orig_message) {
	
	// first, get the list of collections from mr
	
	Element coll_list_message = doc_.createElement("message");
	coll_list_message.setAttribute("lang", page.getAttribute("lang"));
	Element coll_list_request = doc_.createElement("request");
	coll_list_message.appendChild(coll_list_request);
	coll_list_request.setAttribute("type", "describe");
	coll_list_request.setAttribute("info", "collectionList");
	
	Element coll_list_response = (Element)mr_.process(coll_list_message);
	if (coll_list_response==null) {
	    return GSHTML.errorPage("couldn't query the mr about collections");
	}
	
	// second, get the metadata for each collection
	NodeList colls = coll_list_response.getElementsByTagName("collection");
	
	Element metadata_message = doc_.createElement("message");
	metadata_message.setAttribute("lang", page.getAttribute("lang"));
	Element metadata_request = doc_.createElement("request");
	metadata_message.appendChild(metadata_request);
	metadata_request.setAttribute("type", "describe");	
	for (int i=0; i<colls.getLength(); i++) {
	    // get the metadata for each one
	    Element c = (Element)colls.item(i);
	    String name = c.getAttribute("name");
	    
	    metadata_request.setAttribute("to", name); // overwrites the old value
	    Element metadata_response = (Element)mr_.process(metadata_message);
	    Element coll = (Element)metadata_response.getElementsByTagName("collection").item(0);
	    GSXML.mergeMetadataLists(c, coll); // add the metadata to the original response
	}

	//now the full response is in coll_list_response

	// add it in to the page xml tree
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(coll_list_response, "response"), true));

	String stylesheet = GSFile.stylesheetFile(config_, "home.xsl");
	String transformed_result = transformer_.transform(stylesheet,
							   page);    
	return transformed_result;
    	     
    } // homePage

    protected String aboutPage(Element page, Element request) {

	// get the params out
	HashMap params = GSXML.extractParams((Element)GSXML.getNodeByPath(request, "paramList"));
	String coll_name = (String)params.get("c");

	Element coll_about_message = doc_.createElement("message");
	coll_about_message.setAttribute("lang", page.getAttribute("lang"));

	Element coll_about_request = doc_.createElement("request");
	coll_about_message.appendChild(coll_about_request);
	coll_about_request.setAttribute("type", "describe");
	coll_about_request.setAttribute("to", coll_name);

	Element coll_about_response = (Element)mr_.process(coll_about_message);


	// add the response to the page 
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(coll_about_response, "response"), true));

	// process using the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "about.xsl");

	//System.out.println("response="+converter_.getString(page));
	String transformed_result = transformer_.transform(stylesheet,
							   page);    
	return transformed_result;
	
	
    }
}
